/*
 * nandboot.c
 *
 * Copyright (C) 2009 Texas Instruments Incorporated - http://www.ti.com/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation version 2.
 *
 * This program is distributed "as is" WITHOUT ANY WARRANTY of any
 * kind, whether express or implied; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

/** \file   nandboot.c
    \brief  DaVinci Secondary boot-loader, NAND Boot file

    NOTE: THIS FILE IS PROVIDED FOR INITIAL DEMO RELEASE AND MAY BE
          REMOVED AFTER THE DEMO OR THE CONTENTS OF THIS FILE ARE SUBJECT
          TO CHANGE.

    @author    Amit Chatterjee
    @version    0.1 - Modified from ROM Boot Loader
**/

#include <bootloader.h>

#ifdef UBL_USE_NAND_BOOT

#include <nand.h>
#include <armDsp.h>

/* So searches in Block 6-50 */
#define START_BLOCK_NUM 6
#define TRY_BLOCK_NUM 50

extern NAND_DEVICE_INFO gNandDevInfo[];	 /* information about NAND flash */
extern NAND_INFO	gNandInfo;			 /* information about the NAND flash present */
extern NAND_BOOT gNandBoot;				 /* structure for holding details about UBL stored in NAND */
extern Uint32 gEntryPoint;
extern void *ubl_alloc_mem (int size);
Uint32 gDMA = 0;

/* Function to find out where the UBL is and copy to IRAM */
Uint32 NANDCopy() {
	Uint32 count,blockNum;
	Uint32 i;
	Uint32 magicNum;
	Uint32 *rxBuf = (Uint32 *)0x8C000000;		/* receive buffer where the data read from NAND will be stored */
	Uint32 pageSize;
	Uint32 block,page;
	Uint32 readError = E_FAIL;
	Int16  retcode;

	/*rxBuf = (Uint8*)ubl_alloc_mem(MAX_IMAGE_SIZE);*/
	blockNum = START_BLOCK_NUM;

	UARTSendData(22,(Uint8 *) "\r\nStarting NAND Copy\r\n");

NAND_startAgain:
        /* read data about Application from the block 1(to TRY_BLOCK_NUM+1), Page 0 */
	for(count = (blockNum * 64); 
		count <= (TRY_BLOCK_NUM * 64);
		count++) {

		retcode = NANDCheckBadBlock (count/64);
		if (retcode < 0)
		{
			continue;
		}

		readError = NANDReadPage(count, rxBuf,1);
		/* Read Error has occured */
		if(readError != E_PASS)
			continue;

		magicNum = *(((Uint32 *)rxBuf));

		/* Magic number found */
		if((magicNum & 0xFFFFFF00) == MAGIC_NUMBER_VALID) {
			blockNum = count;
			break;
		}
	}

	if(readError != E_PASS)
		return E_FAIL; /* NAND boot failed return to main */

	/* entry point */
	gNandBoot.entryPoint = *(rxBuf + 1);/* The first "long" is entry point for Application */
	gNandBoot.numPage = *(rxBuf + 2);	 /* The second "long" is the number of pages */
	gNandBoot.block = *(rxBuf + 3);	 /* The third "long" is the block where Application is stored in NAND */
	gNandBoot.page = *(rxBuf + 4);	 /* The fourth "long" is the page number where Application is stored in NAND */
	gNandBoot.ldAddress = *(rxBuf + 5);	 /* The fifth "long" is the Application load address */

	/* initialize block and page number to be used for read */
	block = gNandBoot.block;
	page = gNandBoot.page;

	if(gNandInfo.page_size == 2112)
		pageSize = 2048;
	else if(gNandInfo.page_size == 528)
		pageSize = 512;
	else
		pageSize = 256;

	if(magicNum == APP_MAGIC_BIN_IMG) {
		rxBuf = (Uint32 *)gNandBoot.ldAddress;
	}

NAND_retry:
	for(i=1;i<(gNandBoot.numPage + 1);i++) {
	/* if page goes beyond max number of pages increment block number and subtract page number */
		if(page >= gNandInfo.page_cnt) {
			page = 0;
			block++;
		}

		retcode = NANDCheckBadBlock (block);
		if (retcode < 0)
		{
			block++;
			i--;
			continue;
		}

		readError = NANDReadPage((block * 64) + page, rxBuf, 1);	/* Copy the data */

		if(readError != E_PASS) {
			if(magicNum == APP_MAGIC_SAFE) {
				if(blockNum == (TRY_BLOCK_NUM)) {
					return E_FAIL; /* NAND boot failed and return fail to main */
				} else {
					/* Search from for the Magic number in next block */
					blockNum++;
					goto NAND_startAgain;
				}
			} else {
                                magicNum = APP_MAGIC_SAFE;
                                page = gNandBoot.page;
				goto NAND_retry;
			}
		}
		rxBuf += 2048/4;
		page++;
	}

	gEntryPoint = gNandBoot.entryPoint;

	/* Data is already copied to RAM, just set the entry point */
	if(magicNum == APP_MAGIC_BIN_IMG) {
		return E_PASS;
	}

	/*used only if the image is of Srecord format*/
	if(SFMTDecode((Uint8 *)rxBuf, gNandBoot.numPage * pageSize) != E_PASS) {
		return E_FAIL;
	}

	return E_PASS;
}

#define MAX_PAGE_SIZE	(2112) /* Including Spare Area */

#endif /* UBL_USE_NAND_BOOT */
