/*
 * uartboot.c
 *
 * Copyright (C) 2009 Texas Instruments Incorporated - http://www.ti.com/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation version 2.
 *
 * This program is distributed "as is" WITHOUT ANY WARRANTY of any
 * kind, whether express or implied; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

/******************************************************************************/
/* 	FILE	: uartBoot.c	 					 					          */
/*	PURPOSE	: Implementation of UART boot loader                         	  */
/*  PROJECT	: DaVinci ROM boot-loader                                         */
/*  AUTHOR	: Vignesh LA	                                                  */
/*  DATE	: April-25-2005                                                   */
/* 																			  */
/* Revision																	  */
/* Added header details for UART secure boot mode							  */
/* Vignesh - Apr-27-2005													  */
/******************************************************************************/

#include <uart.h>
#include <armDsp.h>
#include <bootloader.h>

extern Uint32 gEntryPoint;
extern Uint32 gSelectUartMenu;
volatile Uint32 gAppStartAddr, gLoopCount, gMagicFlag;
static Uint32 *gStartAddress;

#define VDD3P3V_PWDN        *( unsigned int* )( 0x01c40048 )

extern void *ubl_alloc_mem (int size);

void UARTModeInit() {
	CSL_UART_0_REGS->MCR	=	0x40;	//External LB enabling + Enabling access to TCL, TLR
	CSL_UART_0_REGS->MSR	=	0x0F;
	CSL_UART_0_REGS->SPR	=	0;		//By doing so, FCR determines the FIFO level.

	// SCR enables using FCR to set trigger level
	CSL_UART_0_REGS->SCR	&=	0x3F;

	// FCR and IIR occupy same spot, clear FIFO, enable FIFO.
	CSL_UART_0_REGS->IIR	=	0x07;
	CSL_UART_0_REGS->LCR	&=	0x7F;
	CSL_UART_0_REGS->MCR	&=	0xBF;
	CSL_UART_0_REGS->LCR	=	0x83;

	// EFR in this mode is the same as IIR in standard LCR mode
	// [0:1]CHAR_LENGTH:8bit [2]NB_STOP:1bit [3]PARITY_EN:No
	// [4]PARITY_TYPE1:NoParity[3] [5]PARITY_TYPE2:NoParity
	// [6]BREAK_EN:Normal [7]DIV_EN:Divisor latch enable

	CSL_UART_0_REGS->IIR	&=	0xEF;
	CSL_UART_0_REGS->LCR	=	0x83;

	//DLH in this mode is the same as IER in standard mode
	CSL_UART_0_REGS->IER	=	0;

	//DLL in this mode is the same as RHR in standard mode
	CSL_UART_0_REGS->RHR	=	0x0D;	// very close to the 110b/s on hyper terminal
	CSL_UART_0_REGS->LCR&=0x7F;
	CSL_UART_0_REGS->IER=3;				// Receiver interrupt
	CSL_UART_0_REGS->MDR1=0;			// UART Mode[0:2] 000:UART16x  011:UART13x
	return;
}

Uint32 UARTSendData(Uint32 numBytes, Uint8* seq) {
	Uint32 i, status = 0;
	Uint32 timerStatus = 1;
	for(i=0;i<numBytes;i++) {
		/* Enable Timer one time */
		CSL_INTC1_0_REGS->IRQ1 |= 0x00000001;
		CSL_TMR_1_REGS->TGCR  = 0x00000000;
		CSL_TMR_1_REGS->TIM12 = 0x0;
		CSL_TMR_1_REGS->TCR   = 0x00400040;
		CSL_TMR_1_REGS->TGCR  = 0x00000003;
		do{
			status = (CSL_UART_0_REGS->LSR)&(0x20);
			timerStatus = ((CSL_INTC1_0_REGS->IRQ1)&1);
		} while (!status && timerStatus);

		if(timerStatus == 0)
			return E_TIMEOUT;

		(CSL_UART_0_REGS->RHR) = seq[i];
	}
	return E_PASS;
}

Uint32 UARTSendInt(Uint32 seq) {
	Uint8	data;
	Int		count;

	for(count = 7; count >= 0; count --)
	{
		data = (seq >> (count * 4)) & 0xF;
		if(data > 9)
		{
			data += 7;
		}
		data = data + '0';

		UARTSendData(1, &data);
	}

	return E_PASS;
}

Uint32 UARTCheckSequence(Uint32 numBytes, Uint8* seq) {
	Uint32 i, status = 0;
	Uint32 timerStatus = 1;
	for(i=0;i<numBytes;i++) {
		/* Enable Timer one time */
		CSL_INTC1_0_REGS->IRQ1 |= 0x00000001;
		CSL_TMR_1_REGS->TGCR  = 0x00000000;
		CSL_TMR_1_REGS->TIM12 = 0x0;
		CSL_TMR_1_REGS->TCR   = 0x00400040;
		CSL_TMR_1_REGS->TGCR  = 0x00000003;
		do{
			status = ((CSL_UART_0_REGS->LSR)&1);
			timerStatus = ((CSL_INTC1_0_REGS->IRQ1)&1);
		} while (!status && timerStatus);

		if(timerStatus == 0)
			return E_TIMEOUT;

		if(((CSL_UART_0_REGS->RHR)&0xFF) != seq[i])
			return E_FAIL;
	}
	return E_PASS;
}

Uint32 UARTGetData(Uint32 numBytes, Uint8* seq) {
	Uint32 i;
	Uint32 timerStatus = 1, status = 0;

	for(i=0;i<numBytes;i++) {
		/* Enable Timer one time */
		CSL_INTC1_0_REGS->IRQ1 |= 0x00000001;
		CSL_TMR_1_REGS->TGCR    = 0x00000000;
		CSL_TMR_1_REGS->TIM12   = 0x0;
		CSL_TMR_1_REGS->TCR     = 0x00400040;
		CSL_TMR_1_REGS->TGCR    = 0x00000003;
		do{
			status = ((CSL_UART_0_REGS->LSR)&1);
			timerStatus = ((CSL_INTC1_0_REGS->IRQ1)&1);
		} while (!status && timerStatus);

		if(timerStatus == 0)
			return E_TIMEOUT;

		/* Copy ASCII Data */
		seq[i] = ((CSL_UART_0_REGS->RHR)&0xFF);

		/* Checking for bit 1,2,3,4 for reception Error *///1E->1C
		if(((CSL_UART_0_REGS->LSR)&(0x1C)) != 0)
			return E_FAIL;
	}

	return E_PASS;
}

Uint32 UARTGetHexData(Uint32 numBytes, Uint32* seq) {
	Uint32 i,j;
	Uint32 temp[8];
	Uint32 timerStatus = 1, status = 0;
	Uint32 numLongs, numAsciiChar, shift;
	if(numBytes == 2) {
		numLongs = 1;
		numAsciiChar = 4;
		shift = 12;
	} else {
		numLongs = numBytes/4;
		numAsciiChar = 8;
		shift = 28;
	}

	for(i=0;i<numLongs;i++) {
		seq[i] = 0;
		for(j=0;j<numAsciiChar;j++) {
			/* Enable Timer one time */
			CSL_INTC1_0_REGS->IRQ1 |= 0x00000001;
			CSL_TMR_1_REGS->TGCR  = 0x00000000;
			CSL_TMR_1_REGS->TIM12 = 0x0;
			CSL_TMR_1_REGS->TCR   = 0x00400040;
			CSL_TMR_1_REGS->TGCR  = 0x00000003;
			do{
				status = ((CSL_UART_0_REGS->LSR)&1);
				timerStatus = ((CSL_INTC1_0_REGS->IRQ1)&1);
			} while (!status && timerStatus);

			if(timerStatus == 0)
				return E_TIMEOUT;

			/* Converting ascii to Hex*/
			temp[j] = ((CSL_UART_0_REGS->RHR)&0xFF)-48;

			if(temp[j] > 22)/* To support lower case a,b,c,d,e,f*/
			   temp[j] = temp[j]-39;
			else if(temp[j]>9)/* To support Upper case A,B,C,D,E,F*/
			   temp[j] = temp[j]-7;

			/* Checking for bit 1,2,3,4 for reception Error *///1E->1C
			if(((CSL_UART_0_REGS->LSR)&(0x1C)) != 0)
				return E_FAIL;

			seq[i] |= (temp[j]<<(shift-(j*4)));
		}
	}
	return E_PASS;
}

Uint32 	UARTGetACKSequence() {
	Uint32 error = E_FAIL;

	error = UARTCheckSequence(7,(Uint8*)"    ACK");/*trailing /0 will come along*/
	if(error != E_PASS) {
		return E_FAIL;
	}

	error = UARTGetHexData(4,(Uint32 *)&gLoopCount);
	error |= UARTGetHexData(4,(Uint32 *)&gAppStartAddr);
	error |= UARTGetHexData(4,(Uint32 *)&gMagicFlag);
	error |= UARTCheckSequence(4,(Uint8*)"0000");

	if(!error) {
		/* RBL sets up loop counter and startAddress */
		gEntryPoint = gAppStartAddr;

		/* Verify loopcount */
		if((gLoopCount == 0) || (gLoopCount > MAX_IMAGE_SIZE)) {
			UARTSendData(8,(Uint8*)" BADCNT");/*trailing /0 will come along*/
			return E_FAIL;
		}

		/* Verify gEntryPoint in range */
		if((gEntryPoint < RAM_START_ADDR)||(gEntryPoint > RAM_END_ADDR)) {
			UARTSendData(8,(Uint8*)"BADADDR");/*trailing /0 will come along*/
			return E_FAIL;
		}

		/* Sends BEGIN sequence */
		UARTSendData(8,(Uint8*)"  BEGIN");/*trailing /0 will come along*/
	} else {
		return E_FAIL;
	}

	return E_PASS;
}


void UARTBoot(void) {

	Uint32 error = E_FAIL;

	/* Initialize the variable */
	gStartAddress = 0;

UART_tryAgain:
	/* For any type of error the Serial BOOT MODE is tried continuously */

	/* Initialize UART and TIMER */
	UARTModeInit();

	/* RBL Sends ' BOOTPSP/0' */
	error = UARTSendData(9,(Uint8*)" BOOTPSP");/*trailing /0 will come along*/
	if (error != E_PASS)
		goto UART_tryAgain;

	if(UARTGetACKSequence() != E_PASS)
		goto UART_tryAgain;

	/* Allocate Memory from our internal buffer */
	if (gStartAddress == 0)
	{
		gStartAddress = ubl_alloc_mem (gLoopCount);
	}

	if(E_PASS != UARTGetData(gLoopCount, (Uint8*)gStartAddress))
	{
		UARTSendData(22,(Uint8*)"\r\nUART Receive Error\r\n");
		goto UART_tryAgain;
	}

	UARTSendData(8,(Uint8*)"   DONE");

	/* Leaving UART0 -PINMUX1 Enabled*/
	/* entry point updated*/

	/* Decode Motorola S-record */
	error = SFMTDecode((Uint8 *)gStartAddress, gLoopCount);

	if(error != E_PASS)
	{
		goto UART_tryAgain;
	}

}

Uint32	UARTGetSrecAddr (void)
{
	return ((Uint32)gStartAddress);
}


